/********************************************************************************************
*********************************************************************************************
	"Effortless Backups with Minion Backup", script version 1.0
	by Jen McCown, Jen@MidnightDBA.com, @MidnightDBA, http://www.MidnightDBA.com/Jen 
	(@KenpoDBA)
	Download Minion Backup free: http://www.MinionWare.net/backup/

	Sign up for the Minion by MidnightDBA Newsletter: http://eepurl.com/6M89j

		MinionWare Software - MinionWare.net
		Work like a DBA
		
		MidnightSQL Consulting - MidnightSQL.com
		When your database needs a hero.

	--------------------------------------------------
	  Agenda
	--------------------------------------------------
		Introducing Minion Backup
	
		How to Install
		
		Configure Settings: Minion.BackupSettings
			Ordering DBs
			Excluding Diffs
			Excluding all backups for a DB
		
		Configure Paths: Minion.BackupSettingsPath
			Diff DBs to diff drives
			One active, one inactive
			Backup mirroring
			Server labels
		
		Configure Thresholds
			Copy 
			(Shrinking a log)
			(Cert demo)
		
		See Backup Files
		
		See Logged Info: Minion.BackupLog% tables
		
		(Set up Schedules: Minion.BackupSettingsServer Table)

********************************************************************************************
********************************************************************************************/


/**************************** Introducing Minion Backup ****************************/
/*

		* Free at www.MinionWare.net 
		* Available as Minion Backup Enterprise
		* Integrates with Minion Enterprise
		* Fully functional on its own
		* Main tenants: no extra jobs, works out of the box, highly configurable

*/



/**************************** How to Install ****************************/
/*
		1. Download script, extract to	C:\MinionMaintenance1.1\MinionWare\

		2. Run installer:				.\MinionSetupMaster.ps1 . MBDemo Backup

		3. Set BackupDrive and BackupPath!
			USE MBDemo;
			UPDATE Minion.BackupSettingsPath 
			SET BackupDrive='C:\', BackupPath='SQLBackups\%SoSL%\%DBName%\'
			SELECT * FROM Minion.BackupSettingsPath;

	MB is now up and running, protecting your system. 

	Customize your installation with an update script, add it at the end of InstallOrder.txt, 
	and roll it out to the whole environment, and its consistent. 
	(See: C:\MinionMaintenance1.1\MinionWare\Backup1.4\CustomData.sql )

	Schedule:
	SELECT * FROM Minion.BackupSettingsServer 
	WHERE IsActive=1;
	
	NOTE that you can install on multiple servers:
		.\MinionSetupMaster.ps1 Svr1|Svr2|Svr3|Svr4 MBDemo Backup

	...and/or install multiple modules at once!
		.\MinionSetupMaster.ps1 Svr1|Svr2|Svr3|Svr4 MBDemo Backup|Reindex
		.\MinionSetupMaster.ps1 Svr1|Svr2|Svr3|Svr4 MBDemo All
*/

/**************************** Configure Settings: Minion.BackupSettings ****************************/

USE MBDemo;
GO

---Configuring settings is very easy.

SELECT * FROM Minion.BackupSettings;

EXEC Minion.HELP 'backup', 'Minion.BackupSettings'

/*
MB comes installed with a single setting: one default values row (MinionDefault) 
that provides settings for any database that doesnt have its own specific settings. 
Right now, that's all of them. We'll get into hierarchy later.

EXEC Minion.HELP 'Backup', 'Minion.BackupSettings';

What can we do?
	* Exclude a DB: insert a new row with DBName = <MyDatabaseName>, and Exclude=1.
	
	* Order which backups should go first (using GroupOrder, GroupDBOrder). These
	  settings are weighted, so higher numbers go first.
	
	* Mirror a database backup (Mirror = 1) [] Will require you to set up mirror 
	  location.
	
	* Delete backup files before taking a new backup (DelFileBefore=1, 
	  DelFileBeforeAgree = 1). (NOTE: This isn't ideal, but some shops are SO 
	  tight on space, we have to have this enabled.)

	* Change the amount of the backup history you keep (HistRetDays = 90, for example).

	* Run code before or after each [] database backup (DBPreCode, DBPostCode).

	* Enable dynamic backup tuning (DynamicTuning = 1). This one requires a little 
	  more configuration.

	* Verify the backup (Verify = 1).

	* Set your preferred server in an AG (PreferredServer = NULL for AG preferred 
	  backup server, OR a specific server name). Note that you can set this by backup 
	  type if you wish; fulls from AG preferred, logs from a specific server, etc.

	* Shrink log on log backup, once the log reaches 2048MB, down to 512MB. 
	  (ShrinkLogOnLogBackup=1, ShrinkLogThresholdInMB=2048, ShrinkLogSizeInMB=512).

	* Only allow differential backups on databases that are 10GB or larger, and do a 
	  log backup instead (MinSizeForDiffInGB=10, DiffReplaceAction='Log'. You could 
	  also do a Full backup instead).

	* Copy and/or move backup files once backup is finished. FileAction, FileActionTime

	* Set regular BACKUP options: Encrypt, Name, Expire date, retain days (not what you 
	  think it is; this isn't how often MB will delete backup files, but instead the backup
	  flag RetainDays), etc.

	* Stop (or continue) after checksum error (BackupErrorMgmt='STOP_ON_ERROR', or 
	  'CONTINUE_AFTER_ERROR'.)

	* Mark one row as inactive (IsActive=0). This can be extremely useful for saving alternate
	  backup configurations, or for automating different configs based on live criteria.
	  (Think how this can be combined with precode or postcode programmatically!) []

	* Label each row with a short description and/or purpose (Comment='This is the 
	  default row; do not mark as inactive, ever!')

Settings have a hierarchy: more specific overrides more general. So, a DB-specific
row will override the default row for that DB.

*/

SELECT * FROM Minion.BackupSettings;


------------Customize default settings
UPDATE Minion.BackupSettings
SET HistRetDays = 90, 
	--DBPreCode = 'EXEC Things.ToRunFirst;',
	Verify = 1, 
	ShrinkLogOnLogBackup=1, 
	ShrinkLogThresholdInMB=10240, -- 10GB
	ShrinkLogSizeInMB=2048,
	MinSizeForDiffInGB=10, 
	DiffReplaceAction='Full',
	Comment='This is the default row; do not mark as inactive, ever!'
WHERE DBName = 'MinionDefault';



---------------------------------------------About the Minion.BackupMaster stored procedure: 
/* 
About the Minion.BackupMaster stored procedure: 

	The primary Minion Backup job uses Minion.BackupMaster!
*/	
	EXEC msdb..sp_help_Job @job_name='MinionBackup-AUTO'  

	EXEC Minion.Help 'Backup', 'Minion.BackupMaster';

---------------------------------------------Ordering DBs.

-- Generate backup statements without any order:
EXEC Minion.BackupMaster @DBType = 'User'
	, @BackupType = 'Full'
	, @StmtOnly = 1
	, @Include = NULL
	, @Exclude = NULL
	, @ReadOnly = 1
	, @Debug = 0;


--Insert rows for DB1, DB2, and DB3 (backup type = 'All'), specifying GroupOrder.
INSERT INTO Minion.BackupSettings ([DBName], [Port], [BackupType], [Exclude], [GroupOrder], [GroupDBOrder], [Mirror], [DelFileBefore], [DelFileBeforeAgree], [LogLoc], [HistRetDays], [MinionTriggerPath], [DBPreCode], [DBPostCode], [PushToMinion], [DynamicTuning], [Verify], [PreferredServer], [ShrinkLogOnLogBackup], [ShrinkLogThresholdInMB], [ShrinkLogSizeInMB], [MinSizeForDiffInGB], [DiffReplaceAction], [LogProgress], [FileAction], [FileActionTime], [Encrypt], [Name], [ExpireDateInHrs], [RetainDays], [Descr], [Checksum], [Init], [Format], [CopyOnly], [Skip], [BackupErrorMgmt], [MediaName], [MediaDescription], [IsActive], [Comment]) 
SELECT 'DB1' AS [DBName], NULL AS [Port], 'All' AS [BackupType], 0 AS [Exclude], 10 AS [GroupOrder], 0 AS [GroupDBOrder], 0 AS [Mirror], 0 AS [DelFileBefore], 0 AS [DelFileBeforeAgree], 'Local' AS [LogLoc], 60 AS [HistRetDays], NULL AS [MinionTriggerPath], NULL AS [DBPreCode], NULL AS [DBPostCode], NULL AS [PushToMinion], 1 AS [DynamicTuning], '0' AS [Verify], NULL AS [PreferredServer], 0 AS [ShrinkLogOnLogBackup], NULL AS [ShrinkLogThresholdInMB], NULL AS [ShrinkLogSizeInMB], NULL AS [MinSizeForDiffInGB], NULL AS [DiffReplaceAction], NULL AS [LogProgress], NULL AS [FileAction], NULL AS [FileActionTime], 0 AS [Encrypt], NULL AS [Name], NULL AS [ExpireDateInHrs], NULL AS [RetainDays], NULL AS [Descr], 1 AS [Checksum], 1 AS [Init], 1 AS [Format], NULL AS [CopyOnly], NULL AS [Skip], NULL AS [BackupErrorMgmt], NULL AS [MediaName], NULL AS [MediaDescription], 1 AS [IsActive], NULL AS [Comment]
INSERT INTO Minion.BackupSettings ([DBName], [Port], [BackupType], [Exclude], [GroupOrder], [GroupDBOrder], [Mirror], [DelFileBefore], [DelFileBeforeAgree], [LogLoc], [HistRetDays], [MinionTriggerPath], [DBPreCode], [DBPostCode], [PushToMinion], [DynamicTuning], [Verify], [PreferredServer], [ShrinkLogOnLogBackup], [ShrinkLogThresholdInMB], [ShrinkLogSizeInMB], [MinSizeForDiffInGB], [DiffReplaceAction], [LogProgress], [FileAction], [FileActionTime], [Encrypt], [Name], [ExpireDateInHrs], [RetainDays], [Descr], [Checksum], [Init], [Format], [CopyOnly], [Skip], [BackupErrorMgmt], [MediaName], [MediaDescription], [IsActive], [Comment]) 
SELECT 'DB2' AS [DBName], NULL AS [Port], 'All' AS [BackupType], 0 AS [Exclude], 9 AS [GroupOrder], 0 AS [GroupDBOrder], 0 AS [Mirror], 0 AS [DelFileBefore], 0 AS [DelFileBeforeAgree], 'Local' AS [LogLoc], 60 AS [HistRetDays], NULL AS [MinionTriggerPath], NULL AS [DBPreCode], NULL AS [DBPostCode], NULL AS [PushToMinion], 1 AS [DynamicTuning], '0' AS [Verify], NULL AS [PreferredServer], 0 AS [ShrinkLogOnLogBackup], NULL AS [ShrinkLogThresholdInMB], NULL AS [ShrinkLogSizeInMB], NULL AS [MinSizeForDiffInGB], NULL AS [DiffReplaceAction], NULL AS [LogProgress], NULL AS [FileAction], NULL AS [FileActionTime], 0 AS [Encrypt], NULL AS [Name], NULL AS [ExpireDateInHrs], NULL AS [RetainDays], NULL AS [Descr], 1 AS [Checksum], 1 AS [Init], 1 AS [Format], NULL AS [CopyOnly], NULL AS [Skip], NULL AS [BackupErrorMgmt], NULL AS [MediaName], NULL AS [MediaDescription], 1 AS [IsActive], NULL AS [Comment]
INSERT INTO Minion.BackupSettings ([DBName], [Port], [BackupType], [Exclude], [GroupOrder], [GroupDBOrder], [Mirror], [DelFileBefore], [DelFileBeforeAgree], [LogLoc], [HistRetDays], [MinionTriggerPath], [DBPreCode], [DBPostCode], [PushToMinion], [DynamicTuning], [Verify], [PreferredServer], [ShrinkLogOnLogBackup], [ShrinkLogThresholdInMB], [ShrinkLogSizeInMB], [MinSizeForDiffInGB], [DiffReplaceAction], [LogProgress], [FileAction], [FileActionTime], [Encrypt], [Name], [ExpireDateInHrs], [RetainDays], [Descr], [Checksum], [Init], [Format], [CopyOnly], [Skip], [BackupErrorMgmt], [MediaName], [MediaDescription], [IsActive], [Comment]) 
SELECT 'DB3' AS [DBName], NULL AS [Port], 'All' AS [BackupType], 0 AS [Exclude], 8 AS [GroupOrder], 0 AS [GroupDBOrder], 0 AS [Mirror], 0 AS [DelFileBefore], 0 AS [DelFileBeforeAgree], 'Local' AS [LogLoc], 60 AS [HistRetDays], NULL AS [MinionTriggerPath], NULL AS [DBPreCode], NULL AS [DBPostCode], NULL AS [PushToMinion], 1 AS [DynamicTuning], '0' AS [Verify], NULL AS [PreferredServer], 0 AS [ShrinkLogOnLogBackup], NULL AS [ShrinkLogThresholdInMB], NULL AS [ShrinkLogSizeInMB], NULL AS [MinSizeForDiffInGB], NULL AS [DiffReplaceAction], NULL AS [LogProgress], NULL AS [FileAction], NULL AS [FileActionTime], 0 AS [Encrypt], NULL AS [Name], NULL AS [ExpireDateInHrs], NULL AS [RetainDays], NULL AS [Descr], 1 AS [Checksum], 1 AS [Init], 1 AS [Format], NULL AS [CopyOnly], NULL AS [Skip], NULL AS [BackupErrorMgmt], NULL AS [MediaName], NULL AS [MediaDescription], 1 AS [IsActive], NULL AS [Comment]

SELECT * FROM Minion.BackupSettings;

-- Let's generate those backup statements - they'll be in order!
EXEC Minion.BackupMaster @DBType = 'User'
	, @BackupType = 'Full'
	, @StmtOnly = 1
	, @Include = NULL
	, @Exclude = NULL
	, @ReadOnly = 1
	, @Debug = 0;



---------------------------------------------Let's talk about precedence....
/*
Precedence rules:
	- Once you're working with settings at the DB level, you stay at the DB level. 
	  (Now, DB1, DB2, and DB3 will NEVER retrieve settings from MinionDefault row!)
	- Make sure you have ALL backup types covered!
	- More specific takes precedence over more general.
*/

---------------------------------------------Reset.
DELETE FROM Minion.BackupSettings WHERE DBName <> 'MinionDefault';


---------------------------------------------Excluding Diffs.

INSERT INTO Minion.BackupSettings ([DBName], [Port], [BackupType], [Exclude], [GroupOrder], [GroupDBOrder], [Mirror], [DelFileBefore], [DelFileBeforeAgree], [LogLoc], [HistRetDays], [MinionTriggerPath], [DBPreCode], [DBPostCode], [PushToMinion], [DynamicTuning], [Verify], [PreferredServer], [ShrinkLogOnLogBackup], [ShrinkLogThresholdInMB], [ShrinkLogSizeInMB], [MinSizeForDiffInGB], [DiffReplaceAction], [LogProgress], [FileAction], [FileActionTime], [Encrypt], [Name], [ExpireDateInHrs], [RetainDays], [Descr], [Checksum], [Init], [Format], [CopyOnly], [Skip], [BackupErrorMgmt], [MediaName], [MediaDescription], [IsActive], [Comment]) 
SELECT 'DB3' AS [DBName], NULL AS [Port], 'Diff' AS [BackupType], 0 AS [Exclude], 0 AS [GroupOrder], 0 AS [GroupDBOrder], 0 AS [Mirror], 0 AS [DelFileBefore], 0 AS [DelFileBeforeAgree], 'Local' AS [LogLoc], 60 AS [HistRetDays], NULL AS [MinionTriggerPath], NULL AS [DBPreCode], NULL AS [DBPostCode], NULL AS [PushToMinion], 1 AS [DynamicTuning], '0' AS [Verify], NULL AS [PreferredServer], 0 AS [ShrinkLogOnLogBackup], NULL AS [ShrinkLogThresholdInMB], NULL AS [ShrinkLogSizeInMB], NULL AS [MinSizeForDiffInGB], NULL AS [DiffReplaceAction], NULL AS [LogProgress], NULL AS [FileAction], NULL AS [FileActionTime], 0 AS [Encrypt], NULL AS [Name], NULL AS [ExpireDateInHrs], NULL AS [RetainDays], NULL AS [Descr], 1 AS [Checksum], 1 AS [Init], 1 AS [Format], NULL AS [CopyOnly], NULL AS [Skip], NULL AS [BackupErrorMgmt], NULL AS [MediaName], NULL AS [MediaDescription], 1 AS [IsActive], NULL AS [Comment]

SELECT * FROM Minion.BackupSettings;

UPDATE Minion.BackupSettings 
SET Exclude = 1
WHERE DBName = 'DB3' AND BackupType = 'Diff';

EXEC Minion.BackupMaster @DBType = 'User'
	, @BackupType = 'Diff'
	, @StmtOnly = 1
	, @Include = NULL
	, @Exclude = NULL
	, @ReadOnly = 1
	, @Debug = 0;	

	
-- Make sure all backup types are covered for your database!
INSERT INTO Minion.BackupSettings ([DBName], [Port], [BackupType], [Exclude], [GroupOrder], [GroupDBOrder], [Mirror], [DelFileBefore], [DelFileBeforeAgree], [LogLoc], [HistRetDays], [MinionTriggerPath], [DBPreCode], [DBPostCode], [PushToMinion], [DynamicTuning], [Verify], [PreferredServer], [ShrinkLogOnLogBackup], [ShrinkLogThresholdInMB], [ShrinkLogSizeInMB], [MinSizeForDiffInGB], [DiffReplaceAction], [LogProgress], [FileAction], [FileActionTime], [Encrypt], [Name], [ExpireDateInHrs], [RetainDays], [Descr], [Checksum], [Init], [Format], [CopyOnly], [Skip], [BackupErrorMgmt], [MediaName], [MediaDescription], [IsActive], [Comment]) 
SELECT 'DB3' AS [DBName], NULL AS [Port], 'Full' AS [BackupType], 0 AS [Exclude], 8 AS [GroupOrder], 0 AS [GroupDBOrder], 0 AS [Mirror], 0 AS [DelFileBefore], 0 AS [DelFileBeforeAgree], 'Local' AS [LogLoc], 60 AS [HistRetDays], NULL AS [MinionTriggerPath], NULL AS [DBPreCode], NULL AS [DBPostCode], NULL AS [PushToMinion], 1 AS [DynamicTuning], '0' AS [Verify], NULL AS [PreferredServer], 0 AS [ShrinkLogOnLogBackup], NULL AS [ShrinkLogThresholdInMB], NULL AS [ShrinkLogSizeInMB], NULL AS [MinSizeForDiffInGB], NULL AS [DiffReplaceAction], NULL AS [LogProgress], NULL AS [FileAction], NULL AS [FileActionTime], 0 AS [Encrypt], NULL AS [Name], NULL AS [ExpireDateInHrs], NULL AS [RetainDays], NULL AS [Descr], 1 AS [Checksum], 1 AS [Init], 1 AS [Format], NULL AS [CopyOnly], NULL AS [Skip], NULL AS [BackupErrorMgmt], NULL AS [MediaName], NULL AS [MediaDescription], 1 AS [IsActive], NULL AS [Comment]

INSERT INTO Minion.BackupSettings ([DBName], [Port], [BackupType], [Exclude], [GroupOrder], [GroupDBOrder], [Mirror], [DelFileBefore], [DelFileBeforeAgree], [LogLoc], [HistRetDays], [MinionTriggerPath], [DBPreCode], [DBPostCode], [PushToMinion], [DynamicTuning], [Verify], [PreferredServer], [ShrinkLogOnLogBackup], [ShrinkLogThresholdInMB], [ShrinkLogSizeInMB], [MinSizeForDiffInGB], [DiffReplaceAction], [LogProgress], [FileAction], [FileActionTime], [Encrypt], [Name], [ExpireDateInHrs], [RetainDays], [Descr], [Checksum], [Init], [Format], [CopyOnly], [Skip], [BackupErrorMgmt], [MediaName], [MediaDescription], [IsActive], [Comment]) 
SELECT 'DB3' AS [DBName], NULL AS [Port], 'Log' AS [BackupType], 0 AS [Exclude], 8 AS [GroupOrder], 0 AS [GroupDBOrder], 0 AS [Mirror], 0 AS [DelFileBefore], 0 AS [DelFileBeforeAgree], 'Local' AS [LogLoc], 60 AS [HistRetDays], NULL AS [MinionTriggerPath], NULL AS [DBPreCode], NULL AS [DBPostCode], NULL AS [PushToMinion], 1 AS [DynamicTuning], '0' AS [Verify], NULL AS [PreferredServer], 0 AS [ShrinkLogOnLogBackup], NULL AS [ShrinkLogThresholdInMB], NULL AS [ShrinkLogSizeInMB], NULL AS [MinSizeForDiffInGB], NULL AS [DiffReplaceAction], NULL AS [LogProgress], NULL AS [FileAction], NULL AS [FileActionTime], 0 AS [Encrypt], NULL AS [Name], NULL AS [ExpireDateInHrs], NULL AS [RetainDays], NULL AS [Descr], 1 AS [Checksum], 1 AS [Init], 1 AS [Format], NULL AS [CopyOnly], NULL AS [Skip], NULL AS [BackupErrorMgmt], NULL AS [MediaName], NULL AS [MediaDescription], 1 AS [IsActive], NULL AS [Comment]

SELECT * FROM Minion.BackupSettings;

EXEC Minion.BackupMaster @DBType = 'User'
	, @BackupType = 'Full'
	, @StmtOnly = 1
	, @Include = NULL
	, @Exclude = NULL
	, @ReadOnly = 1
	, @Debug = 0;


EXEC Minion.BackupMaster @DBType = 'User'
	, @BackupType = 'Diff'
	, @StmtOnly = 1
	, @Include = NULL
	, @Exclude = NULL
	, @ReadOnly = 1
	, @Debug = 0;
	


---------------------------------------------Excluding all backups for a DB.
INSERT INTO Minion.BackupSettings ([DBName], [Port], [BackupType], [Exclude], [GroupOrder], [GroupDBOrder], [Mirror], [DelFileBefore], [DelFileBeforeAgree], [LogLoc], [HistRetDays], [MinionTriggerPath], [DBPreCode], [DBPostCode], [PushToMinion], [DynamicTuning], [Verify], [PreferredServer], [ShrinkLogOnLogBackup], [ShrinkLogThresholdInMB], [ShrinkLogSizeInMB], [MinSizeForDiffInGB], [DiffReplaceAction], [LogProgress], [FileAction], [FileActionTime], [Encrypt], [Name], [ExpireDateInHrs], [RetainDays], [Descr], [Checksum], [Init], [Format], [CopyOnly], [Skip], [BackupErrorMgmt], [MediaName], [MediaDescription], [IsActive], [Comment]) 
SELECT 'MBDemo_Jen' AS [DBName], NULL AS [Port], 'All' AS [BackupType], 0 AS [Exclude], 0 AS [GroupOrder], 0 AS [GroupDBOrder], 0 AS [Mirror], 0 AS [DelFileBefore], 0 AS [DelFileBeforeAgree], 'Local' AS [LogLoc], 60 AS [HistRetDays], NULL AS [MinionTriggerPath], NULL AS [DBPreCode], NULL AS [DBPostCode], NULL AS [PushToMinion], 1 AS [DynamicTuning], '0' AS [Verify], NULL AS [PreferredServer], 0 AS [ShrinkLogOnLogBackup], NULL AS [ShrinkLogThresholdInMB], NULL AS [ShrinkLogSizeInMB], NULL AS [MinSizeForDiffInGB], NULL AS [DiffReplaceAction], NULL AS [LogProgress], NULL AS [FileAction], NULL AS [FileActionTime], 0 AS [Encrypt], NULL AS [Name], NULL AS [ExpireDateInHrs], NULL AS [RetainDays], NULL AS [Descr], 1 AS [Checksum], 1 AS [Init], 1 AS [Format], NULL AS [CopyOnly], NULL AS [Skip], NULL AS [BackupErrorMgmt], NULL AS [MediaName], NULL AS [MediaDescription], 1 AS [IsActive], NULL AS [Comment]

UPDATE  Minion.BackupSettings
SET     Exclude = 1
WHERE   DBName = 'MBDemo_Jen'
        AND BackupType = 'All';

SELECT * FROM Minion.BackupSettings;

EXEC Minion.BackupMaster @DBType = 'User'
	, @BackupType = 'Full'
	, @StmtOnly = 1
	, @Include = NULL
	, @Exclude = NULL
	, @ReadOnly = 1
	, @Debug = 0;

EXEC Minion.BackupMaster @DBType = 'User'
	, @BackupType = 'Log'
	, @StmtOnly = 1
	, @Include = NULL
	, @Exclude = NULL
	, @ReadOnly = 1
	, @Debug = 0;

--BONUS: Exclude based on "Exclude", and on wildcards
EXEC Minion.BackupMaster @DBType = 'User'
	, @BackupType = 'Full'
	, @StmtOnly = 1
	, @Include = NULL
	, @Exclude = 'DB%,Minion%'
	, @ReadOnly = 1
	, @Debug = 0;


--BONUS: Exclude based on a list (also using some wildcards):
EXEC Minion.BackupMaster @DBType = 'User'
	, @BackupType = 'Full'
	, @StmtOnly = 1
	, @Include = NULL
	, @Exclude = 'Minion%,DB%,MBDemo_Jen'
	, @ReadOnly = 1
	, @Debug = 0;

---------------------------------------------Reset.
DELETE FROM Minion.BackupSettings WHERE DBName <> 'MinionDefault';
SELECT * FROM Minion.BackupSettings


/**************************** Configure Paths: Minion.BackupSettingsPath ****************************/
/* 
   So Minion.BackupSettings covered backup SETTINGS.
   Minion.BackupSettingsPath covers the paths we'll back up TO.

   EXEC Minion.HELP 'Backup', 'Minion.BackupSettingsPath';
*/
---------------------------------------------Different databases to different drives.

SELECT * FROM Minion.BackupSettingsPath;
UPDATE Minion.BackupSettingsPath SET BackupDrive = 'C:\'
WHERE DBName  = 'MinionDefault';

EXEC Minion.HELP 'backup', 'minion.BackupSettingsPath'

/*
---- IF you're going to have a mirror path for MinionDefault, make sure it's NOT the ID=1 row: 
	INSERT INTO Minion.BackupSettingsPath (DBName, IsMirror, BackupType, BackupLocType, BackupDrive, BackupPath, ServerLabel, RetHrs, FileActionMethod, FileActionMethodFlags, PathOrder, IsActive, AzureCredential, Comment)
	SELECT DBName, 1 AS IsMirror, BackupType, 'NAS' AS BackupLocType, '\\NAS1\' as BackupDrive, 'SQLBackups' AS BackupPath, ServerLabel, RetHrs, FileActionMethod, FileActionMethodFlags, PathOrder, IsActive, AzureCredential, Comment FROM Minion.BackupSettingsPath;

	SELECT * FROM Minion.BackupSettingsPath;

	---------------------------------------------Reset.
	DELETE FROM Minion.BackupSettingsPath WHERE ID > 1;


---- Note that BackupLocType is important if the type is 'URL' (for Azure) 
---- or 'NUL' (not NULL!). Check documentation for different file action methods.

---- MB supports backing up to Azure. With straight T-SQL, it'd look like this: 
	BACKUP DATABASE AdventureWorks2016  
	TO URL = 'https://<mystorageaccountname>.blob.core.windows.net/<mycontainername>/AdventureWorks2016.bak'   
		  WITH CREDENTIAL = '<mycredentialname>'   <<---- AzureCredential column in MB
		 ,COMPRESSION  
		 ,STATS = 5; 
*/

SELECT * FROM Minion.DBMaintInlineTokens

INSERT INTO Minion.BackupSettingsPath ([DBName], [IsMirror], [BackupType], [BackupLocType], [BackupDrive], [BackupPath], [ServerLabel], [RetHrs], [FileActionMethod], [FileActionMethodFlags], [PathOrder], [IsActive], [AzureCredential], [Comment]) 
SELECT 'MBDemo' AS [DBName], 0 AS [IsMirror], 'All' AS [BackupType], 'Local' AS [BackupLocType], 'C:\' AS [BackupDrive], 'SQLBackupsMBDemoALL\' AS [BackupPath], NULL AS [ServerLabel], 24 AS [RetHrs], NULL AS [FileActionMethod], NULL AS [FileActionMethodFlags], 0 AS [PathOrder], 1 AS [IsActive], NULL AS [AzureCredential], 'MinionDev All.' AS [Comment]

INSERT INTO Minion.BackupSettingsPath ([DBName], [IsMirror], [BackupType], [BackupLocType], [BackupDrive], [BackupPath], [ServerLabel], [RetHrs], [FileActionMethod], [FileActionMethodFlags], [PathOrder], [IsActive], [AzureCredential], [Comment]) 
SELECT 'MBDemo' AS [DBName], 0 AS [IsMirror], 'Full' AS [BackupType], 'Local' AS [BackupLocType], 'C:\' AS [BackupDrive], 'SQLBackupsMBDemoFULL\' AS [BackupPath], NULL AS [ServerLabel], 168 AS [RetHrs], NULL AS [FileActionMethod], NULL AS [FileActionMethodFlags], 0 AS [PathOrder], 1 AS [IsActive], NULL AS [AzureCredential], 'MinionDev Full.' AS [Comment]

INSERT INTO Minion.BackupSettingsPath ([DBName], [IsMirror], [BackupType], [BackupLocType], [BackupDrive], [BackupPath], [ServerLabel], [RetHrs], [FileActionMethod], [FileActionMethodFlags], [PathOrder], [IsActive], [AzureCredential], [Comment]) 
SELECT 'MBDemo' AS [DBName], 0 AS [IsMirror], 'Log' AS [BackupType], 'Local' AS [BackupLocType], 'C:\' AS [BackupDrive], 'SQLBackupsMBDemoLOG\' AS [BackupPath], NULL AS [ServerLabel], 168 AS [RetHrs], NULL AS [FileActionMethod], NULL AS [FileActionMethodFlags], 0 AS [PathOrder], 1 AS [IsActive], NULL AS [AzureCredential], 'MinionDev Log.' AS [Comment]
SELECT * FROM Minion.BackupSettingsPath;

/* EXEC Minion.Help 'Backup', 'Minion.BackupStmtGet'; */
EXEC Minion.BackupStmtGet 'MBDemo', 'Full', 1;		
EXEC Minion.BackupStmtGet 'MBDemo', 'Log', 1;

---------------------------------------------Reset.
DELETE FROM Minion.BackupSettingsPath WHERE ID > 1


---------------------------------------------One active, one inactive
-- Let's set up one path to use on the primary server, and one (inactive) on the secondary server:
INSERT INTO Minion.BackupSettingsPath ([DBName], [IsMirror], [BackupType], [BackupLocType], [BackupDrive], [BackupPath], [ServerLabel], [RetHrs], [FileActionMethod], [FileActionMethodFlags], [PathOrder], [IsActive], [AzureCredential], [Comment]) 
SELECT 'MBDemo' AS [DBName], 0 AS [IsMirror], 'All' AS [BackupType], 'NAS' AS [BackupLocType], '\\server1\' AS [BackupDrive], 'SQLBackupsMBDemoALL\' AS [BackupPath], NULL AS [ServerLabel], 48 AS [RetHrs], NULL AS [FileActionMethod], NULL AS [FileActionMethodFlags], 0 AS [PathOrder], 0 AS [IsActive], NULL AS [AzureCredential], 'DR site: MinionDev All.' AS [Comment]

INSERT INTO Minion.BackupSettingsPath ([DBName], [IsMirror], [BackupType], [BackupLocType], [BackupDrive], [BackupPath], [ServerLabel], [RetHrs], [FileActionMethod], [FileActionMethodFlags], [PathOrder], [IsActive], [AzureCredential], [Comment]) 
SELECT 'MBDemo' AS [DBName], 0 AS [IsMirror], 'All' AS [BackupType], 'local' AS [BackupLocType], 'C:\' AS [BackupDrive], 'SQLBackupsMBDemoALL\' AS [BackupPath], NULL AS [ServerLabel], 48 AS [RetHrs], NULL AS [FileActionMethod], NULL AS [FileActionMethodFlags], 0 AS [PathOrder], 1 AS [IsActive], NULL AS [AzureCredential], 'MinionDev All.' AS [Comment]

SELECT * FROM Minion.BackupSettingsPath;
SELECT * FROM Minion.BackupSettings;

---- As your precode, you could enable the DR based (for example) on servername:
-- Set your database precode: 

		--==== THIS WOULD BE PRECODE FOR THE DATABASE (SELECT DBPrecode FROM Minion.BackupSettings): 
		IF (SELECT @@SERVERNAME) <> 'ProductionServer' /* If we are NOT on the prod server, enaable DR site. */
		BEGIN
			UPDATE Minion.BackupSettingsPath SET IsActive=1 WHERE DBName = 'MBDemo' AND Comment LIKE 'DR site%';
			UPDATE Minion.BackupSettingsPath SET IsActive=0 WHERE DBName = 'MBDemo' AND Comment NOT LIKE 'DR site%';
		END
		ELSE		/* ELSE if we ARE on the prod server, enaable the PROD site. */
		BEGIN
			UPDATE Minion.BackupSettingsPath SET IsActive=0 WHERE DBName = 'MBDemo' AND Comment LIKE 'DR site%';
			UPDATE Minion.BackupSettingsPath SET IsActive=1 WHERE DBName = 'MBDemo' AND Comment NOT LIKE 'DR site%';
		END

SELECT * FROM Minion.BackupSettingsPath

--- YOU'RE WELCOME.

SELECT * FROM Minion.BackupSettings
SELECT * FROM Minion.BackupSettingsServer

---------------------------------------------Reset. 
DELETE FROM Minion.BackupSettingsPath WHERE ID > 1;


---------------------------------------------Backup mirroring.

-- Enable backup mirroring for everything:
UPDATE Minion.BackupSettings 
SET Mirror = 1;

SELECT * FROM Minion.BackupSettings;

-- Set up primary and mirror backup path for MBDemo:
-- (If you don't set up BOTH the primary and mirror, it will mess up your backups!)
INSERT INTO Minion.BackupSettingsPath ([DBName], [IsMirror], [BackupType], [BackupLocType], [BackupDrive], [BackupPath], [ServerLabel], [RetHrs], [FileActionMethod], [FileActionMethodFlags], [PathOrder], [IsActive], [AzureCredential], [Comment]) 
SELECT 'MBDemo' AS [DBName], 0 AS [IsMirror], 'All' AS [BackupType], 'Local' AS [BackupLocType], 'C:\' AS [BackupDrive], 'SQLBackupsALL\' AS [BackupPath], NULL AS [ServerLabel], 36 AS [RetHrs], NULL AS [FileActionMethod], NULL AS [FileActionMethodFlags], 0 AS [PathOrder], 1 AS [IsActive], NULL AS [AzureCredential], 'MBDemo primary.' AS [Comment]

INSERT INTO Minion.BackupSettingsPath ([DBName], [IsMirror], [BackupType], [BackupLocType], [BackupDrive], [BackupPath], [ServerLabel], [RetHrs], [FileActionMethod], [FileActionMethodFlags], [PathOrder], [IsActive], [AzureCredential], [Comment]) 
SELECT 'MBDemo' AS [DBName], 1 AS [IsMirror], 'All' AS [BackupType], 'Local' AS [BackupLocType], 'C:\' AS [BackupDrive], 'SQLBackupsMIRROR\' AS [BackupPath], NULL AS [ServerLabel], 36 AS [RetHrs], NULL AS [FileActionMethod], NULL AS [FileActionMethodFlags], 0 AS [PathOrder], 1 AS [IsActive], NULL AS [AzureCredential], 'Mirror.' AS [Comment]


--You can also mirror different DBs and backup types to different locations.
SELECT * FROM Minion.BackupSettingsPath;

EXEC Minion.BackupStmtGet 'MBDemo', 'Full', 1;

EXEC minion.BackupMaster @DBType = 'User', -- varchar(6)
    @BackupType = 'Full', -- varchar(4)
    @StmtOnly = 0, -- bit
    @Include = 'MBDemo' -- varchar(2000)
    
---------------------------------------------Server labels
-- Change the server label and see it change in the path:
UPDATE Minion.BackupSettingsPath 
SET ServerLabel = 'WhoaServerLabel' WHERE DBName = 'MinionDefault';

EXEC Minion.BackupStmtGet 'DB1', 'Full', 1;
EXEC Minion.BackupDB  @DBName = 'DB1', @BackupType = 'Full', @StmtOnly = 1;

-- Actually run the MBDemo backup now! (It will create folders as needed.)
EXEC Minion.BackupMaster @DBType = 'User', @BackupType = 'Full', @StmtOnly = 0, @Include = 'DB1', @Exclude = NULL;


/**************************** Configure Thresholds ****************************/

/* EXEC Minion.HELP 'Backup', 'Minion.BackupTuningThresholds';  

	Backup tuning tutorials: 
	  The Backup Tune-up: http://usergroup.tv/videos/the-backup-tune-up
	  Minion Backup: Advanced Tuning Thresholds https://www.youtube.com/watch?v=0c26H7btwFw
	  Minion Backup: Tuning Threshold Basics https://www.youtube.com/watch?v=FSX-WZRggbE
*/

SELECT * FROM Minion.BackupTuningThresholds;

INSERT INTO Minion.BackupTuningThresholds ([DBName], [BackupType], [SpaceType], [ThresholdMeasure], [ThresholdValue], [NumberOfFiles], [Buffercount], [MaxTransferSize], [Compression], [BlockSize], [BeginTime], [EndTime], [DayOfWeek], [IsActive], [Comment]) 
SELECT 'MinionDefault' AS [DBName], 'All' AS [BackupType], 'DataAndIndex' AS [SpaceType], 'GB' AS [ThresholdMeasure], 20 AS [ThresholdValue], 4 AS [NumberOfFiles], 100 AS [Buffercount], 2097152 AS [MaxTransferSize], 1 AS [Compression], 0 AS [BlockSize], NULL AS [BeginTime], NULL AS [EndTime], NULL AS [DayOfWeek], 1 AS [IsActive], 'Minion default: 20GB+.' AS [Comment]

INSERT INTO Minion.BackupTuningThresholds ([DBName], [BackupType], [SpaceType], [ThresholdMeasure], [ThresholdValue], [NumberOfFiles], [Buffercount], [MaxTransferSize], [Compression], [BlockSize], [BeginTime], [EndTime], [DayOfWeek], [IsActive], [Comment]) 
SELECT 'MinionDefault' AS [DBName], 'All' AS [BackupType], 'DataAndIndex' AS [SpaceType], 'GB' AS [ThresholdMeasure], 50 AS [ThresholdValue], 6 AS [NumberOfFiles], 200 AS [Buffercount], 2097152 AS [MaxTransferSize], 1 AS [Compression], 0 AS [BlockSize], NULL AS [BeginTime], NULL AS [EndTime], NULL AS [DayOfWeek], 1 AS [IsActive], 'Minion default: 50GB+.' AS [Comment]

INSERT INTO Minion.BackupTuningThresholds ([DBName], [BackupType], [SpaceType], [ThresholdMeasure], [ThresholdValue], [NumberOfFiles], [Buffercount], [MaxTransferSize], [Compression], [BlockSize], [BeginTime], [EndTime], [DayOfWeek], [IsActive], [Comment]) 
SELECT 'MinionDefault' AS [DBName], 'All' AS [BackupType], 'DataAndIndex' AS [SpaceType], 'GB' AS [ThresholdMeasure], 100 AS [ThresholdValue], 10 AS [NumberOfFiles], 400 AS [Buffercount], 2097152 AS [MaxTransferSize], 1 AS [Compression], 0 AS [BlockSize], NULL AS [BeginTime], NULL AS [EndTime], NULL AS [DayOfWeek], 1 AS [IsActive], 'Minion default: 100GB+.' AS [Comment]

INSERT INTO Minion.BackupTuningThresholds ([DBName], [BackupType], [SpaceType], [ThresholdMeasure], [ThresholdValue], [NumberOfFiles], [Buffercount], [MaxTransferSize], [Compression], [BlockSize], [BeginTime], [EndTime], [DayOfWeek], [IsActive], [Comment]) 
SELECT 'MinionDefault' AS [DBName], 'All' AS [BackupType], 'DataAndIndex' AS [SpaceType], 'GB' AS [ThresholdMeasure], 150 AS [ThresholdValue], 15 AS [NumberOfFiles], 500 AS [Buffercount], 2097152 AS [MaxTransferSize], 1 AS [Compression], 0 AS [BlockSize], NULL AS [BeginTime], NULL AS [EndTime], NULL AS [DayOfWeek], 1 AS [IsActive], 'Minion default: 150GB+.' AS [Comment]


SELECT * FROM Minion.BackupTuningThresholds;

-- Let's test backup tuning thresholds based on certain sizes:
EXEC Minion.BackupStmtGet 'MBDemo', 'Full', 125;

	---- Wait...why do we have 20 rows instead of 10?	

EXEC Minion.BackupStmtGet 'MBDemo', 'Full', 150

-----Simplify again for the sake of the demo.
UPDATE Minion.BackupSettings 
SET Mirror = 0



--MBDemo
INSERT INTO Minion.BackupTuningThresholds ([DBName], [BackupType], [SpaceType], [ThresholdMeasure], [ThresholdValue], [NumberOfFiles], [Buffercount], [MaxTransferSize], [Compression], [BlockSize], [BeginTime], [EndTime], [DayOfWeek], [IsActive], [Comment]) 
SELECT 'MBDemo' AS [DBName], 'All' AS [BackupType], 'DataAndIndex' AS [SpaceType], 'GB' AS [ThresholdMeasure], 0 AS [ThresholdValue], 1 AS [NumberOfFiles], 0 AS [Buffercount], 0 AS [MaxTransferSize], 1 AS [Compression], 0 AS [BlockSize], NULL AS [BeginTime], NULL AS [EndTime], NULL AS [DayOfWeek], 1 AS [IsActive], 'MBDemo All. DO NOT REMOVE OR CHANGE.' AS [Comment]

INSERT INTO Minion.BackupTuningThresholds ([DBName], [BackupType], [SpaceType], [ThresholdMeasure], [ThresholdValue], [NumberOfFiles], [Buffercount], [MaxTransferSize], [Compression], [BlockSize], [BeginTime], [EndTime], [DayOfWeek], [IsActive], [Comment]) 
SELECT 'MBDemo' AS [DBName], 'All' AS [BackupType], 'DataAndIndex' AS [SpaceType], 'GB' AS [ThresholdMeasure], 20 AS [ThresholdValue], 2 AS [NumberOfFiles], 50 AS [Buffercount], 2097152 AS [MaxTransferSize], 1 AS [Compression], 0 AS [BlockSize], NULL AS [BeginTime], NULL AS [EndTime], NULL AS [DayOfWeek], 1 AS [IsActive], 'MBDemo All. 20GB' AS [Comment]

INSERT INTO Minion.BackupTuningThresholds ([DBName], [BackupType], [SpaceType], [ThresholdMeasure], [ThresholdValue], [NumberOfFiles], [Buffercount], [MaxTransferSize], [Compression], [BlockSize], [BeginTime], [EndTime], [DayOfWeek], [IsActive], [Comment]) 
SELECT 'MBDemo' AS [DBName], 'All' AS [BackupType], 'DataAndIndex' AS [SpaceType], 'GB' AS [ThresholdMeasure], 50 AS [ThresholdValue], 4 AS [NumberOfFiles], 100 AS [Buffercount], 2097152 AS [MaxTransferSize], 1 AS [Compression], 0 AS [BlockSize], NULL AS [BeginTime], NULL AS [EndTime], NULL AS [DayOfWeek], 1 AS [IsActive], 'MBDemo All. 50GB' AS [Comment]

INSERT INTO Minion.BackupTuningThresholds ([DBName], [BackupType], [SpaceType], [ThresholdMeasure], [ThresholdValue], [NumberOfFiles], [Buffercount], [MaxTransferSize], [Compression], [BlockSize], [BeginTime], [EndTime], [DayOfWeek], [IsActive], [Comment]) 
SELECT 'MBDemo' AS [DBName], 'All' AS [BackupType], 'DataAndIndex' AS [SpaceType], 'GB' AS [ThresholdMeasure], 100 AS [ThresholdValue], 7 AS [NumberOfFiles], 150 AS [Buffercount], 2097152 AS [MaxTransferSize], 1 AS [Compression], 0 AS [BlockSize], NULL AS [BeginTime], NULL AS [EndTime], NULL AS [DayOfWeek], 1 AS [IsActive], 'MBDemo All. 100GB' AS [Comment]

INSERT INTO Minion.BackupTuningThresholds ([DBName], [BackupType], [SpaceType], [ThresholdMeasure], [ThresholdValue], [NumberOfFiles], [Buffercount], [MaxTransferSize], [Compression], [BlockSize], [BeginTime], [EndTime], [DayOfWeek], [IsActive], [Comment]) 
SELECT 'MBDemo' AS [DBName], 'All' AS [BackupType], 'DataAndIndex' AS [SpaceType], 'GB' AS [ThresholdMeasure], 150 AS [ThresholdValue], 10 AS [NumberOfFiles], 300 AS [Buffercount], 2097152 AS [MaxTransferSize], 1 AS [Compression], 0 AS [BlockSize], NULL AS [BeginTime], NULL AS [EndTime], NULL AS [DayOfWeek], 1 AS [IsActive], 'MBDemo All. 150GB' AS [Comment]

SELECT * FROM Minion.BackupTuningThresholds

EXEC Minion.BackupStmtGet 'MBDemo', 'Full', 125
EXEC Minion.BackupStmtGet 'Minion', 'Full', 150
EXEC Minion.BackupStmtGet 'Minion', 'Full', 5

---------------------------------------------
-----------END SettingsThreshold Walkthrough-
---------------------------------------------


---------------------------------------------
-----------BEGIN COPY Walkthrough------------
---------------------------------------------
UPDATE Minion.BackupSettings 
SET FileAction = 'Copy',
FileActionTime = 'AfterBatch' -- 'AfterBackup'		

SELECT * FROM Minion.BackupSettings;

---------------------------------------------Reset.
DELETE Minion.BackupSettingsPath WHERE ID > 1;

--Copy 1
INSERT INTO Minion.BackupSettingsPath ([DBName], [IsMirror], [BackupType], [BackupLocType], [BackupDrive], [BackupPath], [ServerLabel], [RetHrs], [FileActionMethod], [FileActionMethodFlags], [PathOrder], [IsActive], [AzureCredential], [Comment]) 
SELECT 'MinionDefault' AS [DBName], 0 AS [IsMirror], 'Copy' AS [BackupType], 'Local' AS [BackupLocType], 'C:\' AS [BackupDrive], 'SQLBackupsMinionDefaultCOPY1\' AS [BackupPath], NULL AS [ServerLabel], 48 AS [RetHrs], 'ROBOCOPY' AS [FileActionMethod], NULL AS [FileActionMethodFlags], 0 AS [PathOrder], 1 AS [IsActive], NULL AS [AzureCredential], 'MinionDefault Copy.' AS [Comment]

SELECT * FROM Minion.BackupSettingsPath;

EXEC Minion.BackupMaster 
	@DBType = 'User', @BackupType = 'Full', @StmtOnly = 0, @Include = 'MBDemo', @Exclude = NULL

SELECT * FROM Minion.BackupLogDetails ORDER BY ID Desc

--Copy 2
INSERT INTO Minion.BackupSettingsPath ([DBName], [IsMirror], [BackupType], [BackupLocType], [BackupDrive], [BackupPath], [ServerLabel], [RetHrs], [FileActionMethod], [FileActionMethodFlags], [PathOrder], [IsActive], [AzureCredential], [Comment]) 
SELECT 'MinionDefault' AS [DBName], 0 AS [IsMirror], 'Copy' AS [BackupType], 'Local' AS [BackupLocType], 'C:\' AS [BackupDrive], 'SQLBackupsMinionDefaultCOPY2\' AS [BackupPath], NULL AS [ServerLabel], 24 AS [RetHrs], 'ROBOCOPY' AS [FileActionMethod], NULL AS [FileActionMethodFlags], 0 AS [PathOrder], 1 AS [IsActive], NULL AS [AzureCredential], 'MinionDefault Copy.' AS [Comment]

SELECT * FROM Minion.BackupSettingsPath
EXEC Minion.BackupMaster @DBType = 'User', @BackupType = 'Full', @StmtOnly = 0, @Include = 'MBDemo', @Exclude = NULL


------ServerLabel is still really cool!!!!
UPDATE Minion.BackupSettingsPath
SET ServerLabel = 'MinionDev.MinionWare.NET'
WHERE BackupPath = 'SQLBackupsMinionDefaultCOPY1\'

SELECT * FROM Minion.BackupSettingsPath
EXEC Minion.BackupMaster @DBType = 'User', @BackupType = 'Full', @StmtOnly = 0, @Include = 'MinionDev', @Exclude = NULL


------------------------------------Reset.
UPDATE Minion.BackupSettings 
SET FileAction = NULL,
FileActionTime = NULL;
---------------------------------------------
-----------END COPY Walkthrough--------------
---------------------------------------------


/**************************** Shrink Log ****************************/
--Grow the log to 2GB
ALTER DATABASE [MBDemo] MODIFY FILE ( NAME = N'MBDemoLog', SIZE = 2048MB );
GO

DBCC SQLPERF(LOGSPACE);

UPDATE Minion.BackupSettings 
SET ShrinkLogOnLogBackup = 1,
	ShrinkLogThresholdInMB = 5000,
	ShrinkLogSizeInMB = 10;

--This one won't shrink.
EXEC Minion.BackupMaster @DBType = 'User', @BackupType = 'Log', @StmtOnly = 0, @Include = 'MBDemo', @Exclude = NULL

DBCC SQLPERF(LOGSPACE)

----Query the log stuff out of the Details
SELECT  ExecutionDateTime
      , ShrinkLogOnLogBackup
      , ShrinkLogThresholdInMB
      , ShrinkLogSizeInMB
      , PreBackupLogSizeInMB
      , PreBackupLogUsedPct
      , PostBackupLogSizeInMB
      , PostBackupLogUsedPct
      , PreBackupLogReuseWait
      , PostBackupLogReuseWait
      , VLFs
FROM    Minion.BackupLogDetails
ORDER BY ID DESC


--Alter settings to shrink if it's past 1GB
UPDATE Minion.BackupSettings 
SET ShrinkLogOnLogBackup = 1,
	ShrinkLogThresholdInMB = 1024,
	ShrinkLogSizeInMB = 512;
GO

--This one WILL shrink.
Minion.BackupMaster @DBType = 'User', @BackupType = 'Log', @StmtOnly = 0, @Include = 'MBDemo', @Exclude = NULL

DBCC SQLPERF(LOGSPACE)

----Query the log stuff out of the Details
SELECT executionDateTime, DBName, BackupType, ShrinkLogOnLogBackup, ShrinkLogThresholdInMB, ShrinkLogSizeInMB, 
PreBackupLogSizeInMB, PreBackupLogUsedPct, PostBackupLogSizeInMB, PostBackupLogUsedPct, PreBackupLogReuseWait, PostBackupLogReuseWait, VLFs
FROM Minion.BackupLogDetails
ORDER BY ID DESC

---------------------------------------------
-----------END ShrinkLog Walkthrough---------
---------------------------------------------


---------------------------------------------Cert backups.

---------------------------------------------
-----------BEGIN Cert Backup Walkthrough-----
---------------------------------------------
SELECT * FROM Minion.BackupCert; 


-- Save the password to give to the cert backup file.
INSERT Minion.BackupCert (CertType, CertPword, BackupCert)
SELECT 'ServerCert', Minion.EncryptTxt ('ThisIsAGoodPword!!!111!!!'), 1;


----The DBName doesn't matter. Use "MinionDefault" if you like, to represent that it's universal.
INSERT INTO Minion.BackupSettingsPath ([DBName], [IsMirror], [BackupType], [BackupLocType], [BackupDrive], [BackupPath], [ServerLabel], [RetHrs], [FileActionMethod], [FileActionMethodFlags], [PathOrder], [IsActive], [AzureCredential], [Comment]) 
SELECT 'MinionDefault' AS [DBName], 0 AS [IsMirror], 'ServerCert' AS [BackupType], 'Local' AS [BackupLocType], 'C:\' AS [BackupDrive], 'SQLBackupsMinionDefaultServerCERT1\' AS [BackupPath], NULL AS [ServerLabel], 2016 AS [RetHrs], NULL AS [FileActionMethod], NULL AS [FileActionMethodFlags], 0 AS [PathOrder], 1 AS [IsActive], NULL AS [AzureCredential], 'ServerCert.' AS [Comment]

SELECT * FROM Minion.BackupSettingsPath;


---- Run the system backups; server cert backups happen automatically!
EXEC Minion.BackupMaster @DBType = 'System', @BackupType = 'Full', @StmtOnly = 0, @Include = 'master', @Exclude = NULL




--Add a 2nd location for the cert backup.
INSERT INTO Minion.BackupSettingsPath ([DBName], [IsMirror], [BackupType], [BackupLocType], [BackupDrive], [BackupPath], [ServerLabel], [RetHrs], [FileActionMethod], [FileActionMethodFlags], [PathOrder], [IsActive], [AzureCredential], [Comment]) 
SELECT 'MinionDefault' AS [DBName], 0 AS [IsMirror], 'ServerCert' AS [BackupType], 'Local' AS [BackupLocType], '\\Minioncon\c$\' AS [BackupDrive], 'CertBackups\' AS [BackupPath], NULL AS [ServerLabel], 2016 AS [RetHrs], NULL AS [FileActionMethod], NULL AS [FileActionMethodFlags], 0 AS [PathOrder], 1 AS [IsActive], NULL AS [AzureCredential], 'ServerCert.' AS [Comment]

EXEC Minion.BackupMaster @DBType = 'System', @BackupType = 'Full', @StmtOnly = 0, @Include = NULL, @Exclude = NULL

----Now let's see what a failure looks like.
INSERT INTO Minion.BackupSettingsPath ([DBName], [IsMirror], [BackupType], [BackupLocType], [BackupDrive], [BackupPath], [ServerLabel], [RetHrs], [FileActionMethod], [FileActionMethodFlags], [PathOrder], [IsActive], [AzureCredential], [Comment]) 
SELECT 'MinionDefault' AS [DBName], 0 AS [IsMirror], 'ServerCert' AS [BackupType], 'Local' AS [BackupLocType], '\\Minioncon\KB\' AS [BackupDrive], 'CertBackups\' AS [BackupPath], NULL AS [ServerLabel], 2016 AS [RetHrs], NULL AS [FileActionMethod], NULL AS [FileActionMethodFlags], 0 AS [PathOrder], 1 AS [IsActive], NULL AS [AzureCredential], 'ServerCert.' AS [Comment]

EXEC Minion.BackupMaster @DBType = 'System', @BackupType = 'Full', @StmtOnly = 0, @Include = NULL, @Exclude = NULL

SELECT * FROM Minion.BackupLog
ORDER BY ID DESC;

SELECT * FROM Minion.BackupLogDetails
ORDER BY ID DESC

SELECT * FROM Minion.BackupFiles
ORDER BY ID DESC

---------------------------------------------
-----------END Cert Backup Walkthrough-------
---------------------------------------------




/**************************** See Backup Files ****************************/
/* 
	Minion.BackupFiles: A log of all backup files (whether they originate from a backup, a copy, or a move).

	EXEC Minion.HELP 'backup', 'Minion.BackupFiles'; 
*/

SELECT * FROM Minion.BackupFiles;


/**************************** See Logged Info: Minion.BackupLog% tables ****************************/

---------------------------------------------
-----------BEGIN Monitor Process Walkthrough-
---------------------------------------------

---Watch the progress of this DB as it backs up.
---Open a new window for the Status.

--Only run this if you need to reset the entire demo.
----TRUNCATE TABLE Minion.BackupLog
----TRUNCATE TABLE Minion.BackupLogDetails
----TRUNCATE TABLE Minion.BackupFiles

SELECT  *
FROM    Minion.BackupLog
ORDER BY ID DESC

SELECT  *
FROM    Minion.BackupLogDetails
ORDER BY ID DESC

EXEC Minion.BackupMaster @DBType = 'User', @BackupType = 'Full', @StmtOnly = 0, @Include = NULL, @Exclude = NULL


EXEC Minion.BackupMaster @DBType = 'User', @BackupType = 'Full', @StmtOnly = 0, @Include = 'MBDemo,Demo', @Exclude = NULL



---------------------------------------------
-----------END Monitor Process Walkthrough---
---------------------------------------------


/**************************** Set up Schedules: Minion.BackupSettingsServer Table ****************************/

---------------------------------------------
-----------BEGIN Scheduler Walkthrough-------
---------------------------------------------
/* EXEC Minion.HELP 'Backup', 'Minion.BackupSettingsServer';  */

SELECT * FROM Minion.BackupSettingsServer

UPDATE Minion.BackupSettingsServer
SET IsActive=1, BackupType='Full'
WHERE ID = 6;

---------------------------------------------
-----------END Scheduler Walkthrough---------
---------------------------------------------


Minion.BackupRestoreDB @DBName = 'MinionDev', @BackupType = 'Full', @BackupLoc = 'Backup', @StmtOnly = 1

Minion.HELP 'Backup', 'How to: Copy files after backup (single and multiple locations) '






-- Bonus troubleshooting: For failed backups, check that powershell get-execcutionpolicy = RemoteSigned.
-- And check for problems in BackupSettingsPath, BackupSettings, BackupSettingsThresholds


/**
use master;
go
DROP DATABASE MBDemo
GO

CREATE DATABASE MBDemo
ON
(
    NAME = MBDemo,
    FILENAME = 'C:\Program Files\Microsoft SQL Server\MSSQL12.MSSQLSERVER\MSSQL\DATA\MBDemo2.mdf',
    SIZE = 512,
    MAXSIZE = UNLIMITED,
    FILEGROWTH = 512
)
LOG ON
(
    NAME = MBDemoLog,
    FILENAME = 'C:\Program Files\Microsoft SQL Server\MSSQL12.MSSQLSERVER\MSSQL\DATA\MBDemoLog2.ldf',
    SIZE = 256,
    MAXSIZE = UNLIMITED,
    FILEGROWTH = 512
);

 CREATE DATABASE DB1;
 CREATE DATABASE DB2;
 CREATE DATABASE DB3;

-- */